/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_THUMBNAILER_SERVICE_H__
#define __PHOSH_THUMBNAILER_SERVICE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for mobi.phosh.Thumbnailer */

#define PT_IMPL_TYPE_THUMBNAILER (pt_impl_thumbnailer_get_type ())
#define PT_IMPL_THUMBNAILER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PT_IMPL_TYPE_THUMBNAILER, PtImplThumbnailer))
#define PT_IMPL_IS_THUMBNAILER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PT_IMPL_TYPE_THUMBNAILER))
#define PT_IMPL_THUMBNAILER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PT_IMPL_TYPE_THUMBNAILER, PtImplThumbnailerIface))

struct _PtImplThumbnailer;
typedef struct _PtImplThumbnailer PtImplThumbnailer;
typedef struct _PtImplThumbnailerIface PtImplThumbnailerIface;

struct _PtImplThumbnailerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_stop_thumbnailing) (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_options);

  gboolean (*handle_thumbnail_directory) (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_directory,
    GVariant *arg_options);

  gboolean (*handle_thumbnail_files) (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_files,
    GVariant *arg_options);

  void (*thumbnailing_done) (
    PtImplThumbnailer *object,
    GVariant *arg_thumbnails,
    GVariant *arg_options);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PtImplThumbnailer, g_object_unref)
#endif

GType pt_impl_thumbnailer_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *pt_impl_thumbnailer_interface_info (void);
guint pt_impl_thumbnailer_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void pt_impl_thumbnailer_complete_thumbnail_directory (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation);

void pt_impl_thumbnailer_complete_thumbnail_files (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation);

void pt_impl_thumbnailer_complete_stop_thumbnailing (
    PtImplThumbnailer *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void pt_impl_thumbnailer_emit_thumbnailing_done (
    PtImplThumbnailer *object,
    GVariant *arg_thumbnails,
    GVariant *arg_options);



/* D-Bus method calls: */
void pt_impl_thumbnailer_call_thumbnail_directory (
    PtImplThumbnailer *proxy,
    const gchar *arg_directory,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean pt_impl_thumbnailer_call_thumbnail_directory_finish (
    PtImplThumbnailer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean pt_impl_thumbnailer_call_thumbnail_directory_sync (
    PtImplThumbnailer *proxy,
    const gchar *arg_directory,
    GVariant *arg_options,
    GCancellable *cancellable,
    GError **error);

void pt_impl_thumbnailer_call_thumbnail_files (
    PtImplThumbnailer *proxy,
    const gchar *const *arg_files,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean pt_impl_thumbnailer_call_thumbnail_files_finish (
    PtImplThumbnailer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean pt_impl_thumbnailer_call_thumbnail_files_sync (
    PtImplThumbnailer *proxy,
    const gchar *const *arg_files,
    GVariant *arg_options,
    GCancellable *cancellable,
    GError **error);

void pt_impl_thumbnailer_call_stop_thumbnailing (
    PtImplThumbnailer *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean pt_impl_thumbnailer_call_stop_thumbnailing_finish (
    PtImplThumbnailer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean pt_impl_thumbnailer_call_stop_thumbnailing_sync (
    PtImplThumbnailer *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PT_IMPL_TYPE_THUMBNAILER_PROXY (pt_impl_thumbnailer_proxy_get_type ())
#define PT_IMPL_THUMBNAILER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PT_IMPL_TYPE_THUMBNAILER_PROXY, PtImplThumbnailerProxy))
#define PT_IMPL_THUMBNAILER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PT_IMPL_TYPE_THUMBNAILER_PROXY, PtImplThumbnailerProxyClass))
#define PT_IMPL_THUMBNAILER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PT_IMPL_TYPE_THUMBNAILER_PROXY, PtImplThumbnailerProxyClass))
#define PT_IMPL_IS_THUMBNAILER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PT_IMPL_TYPE_THUMBNAILER_PROXY))
#define PT_IMPL_IS_THUMBNAILER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PT_IMPL_TYPE_THUMBNAILER_PROXY))

typedef struct _PtImplThumbnailerProxy PtImplThumbnailerProxy;
typedef struct _PtImplThumbnailerProxyClass PtImplThumbnailerProxyClass;
typedef struct _PtImplThumbnailerProxyPrivate PtImplThumbnailerProxyPrivate;

struct _PtImplThumbnailerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PtImplThumbnailerProxyPrivate *priv;
};

struct _PtImplThumbnailerProxyClass
{
  GDBusProxyClass parent_class;
};

GType pt_impl_thumbnailer_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PtImplThumbnailerProxy, g_object_unref)
#endif

void pt_impl_thumbnailer_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PtImplThumbnailer *pt_impl_thumbnailer_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PtImplThumbnailer *pt_impl_thumbnailer_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void pt_impl_thumbnailer_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PtImplThumbnailer *pt_impl_thumbnailer_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PtImplThumbnailer *pt_impl_thumbnailer_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PT_IMPL_TYPE_THUMBNAILER_SKELETON (pt_impl_thumbnailer_skeleton_get_type ())
#define PT_IMPL_THUMBNAILER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PT_IMPL_TYPE_THUMBNAILER_SKELETON, PtImplThumbnailerSkeleton))
#define PT_IMPL_THUMBNAILER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PT_IMPL_TYPE_THUMBNAILER_SKELETON, PtImplThumbnailerSkeletonClass))
#define PT_IMPL_THUMBNAILER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PT_IMPL_TYPE_THUMBNAILER_SKELETON, PtImplThumbnailerSkeletonClass))
#define PT_IMPL_IS_THUMBNAILER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PT_IMPL_TYPE_THUMBNAILER_SKELETON))
#define PT_IMPL_IS_THUMBNAILER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PT_IMPL_TYPE_THUMBNAILER_SKELETON))

typedef struct _PtImplThumbnailerSkeleton PtImplThumbnailerSkeleton;
typedef struct _PtImplThumbnailerSkeletonClass PtImplThumbnailerSkeletonClass;
typedef struct _PtImplThumbnailerSkeletonPrivate PtImplThumbnailerSkeletonPrivate;

struct _PtImplThumbnailerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PtImplThumbnailerSkeletonPrivate *priv;
};

struct _PtImplThumbnailerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType pt_impl_thumbnailer_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PtImplThumbnailerSkeleton, g_object_unref)
#endif

PtImplThumbnailer *pt_impl_thumbnailer_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_THUMBNAILER_SERVICE_H__ */
